package javax.swing;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.beans.PropertyChangeListener;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.beans.PropertyChangeEvent;
import javax.swing.event.SwingPropertyChangeSupport;
import java.io.Serializable;

public abstract class AbstractAction implements Action, Cloneable, Serializable
{
    private static Boolean RECONFIGURE_ON_NULL;
    protected boolean enabled;
    private transient ArrayTable arrayTable;
    protected SwingPropertyChangeSupport changeSupport;
    
    static boolean shouldReconfigure(PropertyChangeEvent propertyChangeEvent) {
        if (propertyChangeEvent.getPropertyName() == null) {
            synchronized (AbstractAction.class) {
                if (AbstractAction.RECONFIGURE_ON_NULL == null) {
                    AbstractAction.RECONFIGURE_ON_NULL = Boolean.valueOf((String)AccessController.doPrivileged(new GetPropertyAction("swing.actions.reconfigureOnNull", "false")));
                }
                return (boolean)AbstractAction.RECONFIGURE_ON_NULL;
            }
        }
        return false;
    }
    
    static void setEnabledFromAction(JComponent component, Action action) {
        component.setEnabled(action == null || action.isEnabled());
    }
    
    static void setToolTipTextFromAction(JComponent component, Action action) {
        component.setToolTipText((action != null) ? ((String)action.getValue("ShortDescription")) : null);
    }
    
    static boolean hasSelectedKey(Action action) {
        return action != null && action.getValue("SwingSelectedKey") != null;
    }
    
    static boolean isSelected(Action action) {
        return Boolean.TRUE.equals(action.getValue("SwingSelectedKey"));
    }
    
    public AbstractAction() {
        this.enabled = true;
    }
    
    public AbstractAction(String s) {
        this.enabled = true;
        this.putValue("Name", s);
    }
    
    public AbstractAction(String s, Icon icon) {
        this(s);
        this.putValue("SmallIcon", icon);
    }
    
    @Override
    public Object getValue(String s) {
        if (s == "enabled") {
            return Boolean.valueOf(this.enabled);
        }
        if (this.arrayTable == null) {
            return null;
        }
        return this.arrayTable.get(s);
    }
    
    @Override
    public void putValue(String s, Object value) {
        Object o = null;
        if (s == "enabled") {
            if (value == null || !(value instanceof Boolean)) {
                value = Boolean.valueOf(false);
            }
            o = Boolean.valueOf(this.enabled);
            this.enabled = (boolean)value;
        }
        else {
            if (this.arrayTable == null) {
                this.arrayTable = new ArrayTable();
            }
            if (this.arrayTable.containsKey(s)) {
                o = this.arrayTable.get(s);
            }
            if (value == null) {
                this.arrayTable.remove(s);
            }
            else {
                this.arrayTable.put(s, value);
            }
        }
        this.firePropertyChange(s, o, value);
    }
    
    @Override
    public boolean isEnabled() {
        return this.enabled;
    }
    
    @Override
    public void setEnabled(boolean b) {
        int enabled = this.enabled;
        if (enabled != (b ? 1 : 0)) {
            this.enabled = b;
            this.firePropertyChange("enabled", Boolean.valueOf((boolean)enabled), Boolean.valueOf(b));
        }
    }
    
    public Object[] getKeys() {
        if (this.arrayTable == null) {
            return null;
        }
        Object[] array = new Object[this.arrayTable.size()];
        this.arrayTable.getKeys(array);
        return array;
    }
    
    protected void firePropertyChange(String propertyName, Object oldValue, Object o) {
        if (this.changeSupport == null || (oldValue != null && o != null && oldValue.equals(o))) {
            return;
        }
        this.changeSupport.firePropertyChange(propertyName, oldValue, o);
        return;
    }
    
    @Override
    public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
        if (this.changeSupport == null) {
            this.changeSupport = new SwingPropertyChangeSupport(this);
        }
        this.changeSupport.addPropertyChangeListener(listener);
    }
    
    @Override
    public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
        if (this.changeSupport == null) {
            return;
        }
        this.changeSupport.removePropertyChangeListener(listener);
        return;
    }
    
    public synchronized PropertyChangeListener[] getPropertyChangeListeners() {
        if (this.changeSupport == null) {
            return new PropertyChangeListener[0];
        }
        return this.changeSupport.getPropertyChangeListeners();
    }
    
    @Override
    protected Object clone() throws CloneNotSupportedException {
        AbstractAction abstractAction = (AbstractAction)super.clone();
        synchronized (this) {
            if (this.arrayTable != null) {
                abstractAction.arrayTable = (ArrayTable)this.arrayTable.clone();
            }
        }
        return abstractAction;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        ArrayTable.writeArrayTable(objectOutputStream, this.arrayTable);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
        objectInputStream.defaultReadObject();
        for (int i = objectInputStream.readInt() - 1; i >= 0; --i) {
            this.putValue((String)objectInputStream.readObject(), objectInputStream.readObject());
        }
    }
}
